typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;

typedef volatile u8 vu8;
typedef volatile u16 vu16;
typedef volatile u32 vu32;

typedef signed char s8;
typedef signed short s16;
typedef signed int s32;

typedef u16 COLOR;

// Memory
#define MEM_EWRAM 0x02000000
#define MEM_IWRAM 0x03000000
#define MEM_IO 0x04000000
#define MEM_PAL 0x05000000
#define MEM_VRAM 0x06000000
#define MEM_OAM 0x07000000
#define MEM_ROM 0x08000000
#define MEM_SRAM 0x0E000000

// Bitmap mode video memory
#define vid_mem ((COLOR*) MEM_VRAM)
#define vid_mem_32 ((u32*) MEM_VRAM)

// SRAM pointer
#define sram_mem ((u8*) MEM_SRAM)

// Register base address
#define REG_BASE MEM_IO

// Registers
#define REG_DISPCNT (*(vu32*) MEM_IO)
#define REG_VCOUNT (*(vu16*) 0x04000006)

// Display control flags
#define DCNT_MODE0 0x0000
#define DCNT_MODE1 0x0001
#define DCNT_MODE2 0x0002
#define DCNT_MODE3 0x0003
#define DCNT_MODE4 0x0004
#define DCNT_MODE5 0x0005
#define DCNT_GB 0x0008
#define DCNT_PAGE 0x0010
#define DCNT_OAM_HBL 0x0020
#define DCNT_OBJ_2D 0
#define DCNT_OBJ_1D 0x0040
#define DCNT_BLANK 0x0080
#define DCNT_BG0 0x0100
#define DCNT_BG1 0x0200
#define DCNT_BG2 0x0400
#define DCNT_BG3 0x0800
#define DCNT_OBJ 0x1000
#define DCNT_WIN0 0x2000
#define DCNT_WIN1 0x4000
#define DCNT_WINOBJ 0x8000

// Input
extern u16 __key_curr, __key_prev;

#define REG_KEYINPUT *(vu16*)(REG_BASE+0x0130)

#define KEY_A 0x0001
#define KEY_B 0x0002
#define KEY_SELECT 0x0004
#define KEY_START 0x0008
#define KEY_RIGHT 0x0010
#define KEY_LEFT 0x0020
#define KEY_UP 0x0040
#define KEY_DOWN 0x0080
#define KEY_R 0x0100
#define KEY_L 0x0200

#define KEY_ACCEPT 0x0009
#define KEY_CANCEL 0x0002
#define KEY_RESET 0x030C

#define KEY_FIRE 0x0003
#define KEY_SPECIAL 0x000C
#define KEY_DIR 0x00F0
#define KEY_SHOULDER 0x0300

#define KEY_ANY 0x03FF

#define KEY_MASK 0x03FF

#define KEY_DOWN_NOW(key) (~(REG_KEYINPUT) & key)

// Sound control registers
#define REG_SNDCNT *(vu32*) (REG_BASE + 0x0080)
#define REG_SNDDMGCNT *(vu16*) (REG_BASE + 0x0080)
#define REG_SNDDSCNT *(vu16*) (REG_BASE + 0x0082)
#define REG_SNDSTAT *(vu16*) (REG_BASE + 0x0084)
#define REG_SNDBIAS *(vu16*) (REG_BASE + 0x0088)

// Sound manager flags
#define SDMG_SQR1 0x01
#define SDMG_SQR2 0x02
#define SDMG_WAVE 0x04
#define SDMG_NOISE 0x08

#define SDMG_BUILD(_lmode, _rmode, _lvol, _rvol) \
    ((_lvol & 7) | ((_rvol & 7) << 4) | (_lmode << 8) | (_rmode << 12))

#define SDMG_BUILD_LR(_mode, _vol) SDMG_BUILD(_mode, _mode, _vol, _vol)

// Sound DSound flags
#define SDS_DMG25 0
#define SDS_DMG50 0x0001
#define SDS_DMG100 0x0002
#define SDS_A50 0
#define SDS_A100 0x0004
#define SDS_B50 0
#define SDS_B100 0x0008
#define SDS_AR 0x0100
#define SDS_AL 0x0200
#define SDS_ATMR0 0
#define SDS_ATMR1 0x0400
#define SDS_ARESET 0x0800
#define SDS_BR 0x1000
#define SDS_BL 0x2000
#define SDS_BTMR0 0
#define SDS_BTMR1 0x4000
#define SDS_BRESET 0x8000

// Sound status flags
#define SSTAT_SQR1 0x0001
#define SSTAT_SQR2 0x0002
#define SSTAT_WAVE 0x0004
#define SSTAT_NOISE 0x0008
#define SSTAT_DISABLE 0
#define SSTAT_ENABLE 0x0080

// Sound square wave channel 1
#define REG_SND1SWEEP *(vu16*) (REG_BASE + 0x0060)
#define REG_SND1CNT *(vu16*) (REG_BASE + 0x0062)
#define REG_SND1FREQ *(vu16*) (REG_BASE + 0x0064)

// Sound channel flags
#define SFREQ_HOLD 0
#define SFREQ_TIMED 0x4000
#define SFREQ_RESET 0x8000

#define SFREQ_RATE_MASK 0x07FF
#define SFREQ_RATE_SHIFT 0
#define SFREQ_RATE(n) (n << SFREQ_RATE_SHIFT)

#define SFREQ_BUILD(rate, timed, reset) \
    ((rate & 0x7FF) | (timed << 14) | (reset << 15))

// Sound sweep flags
#define SSW_INC 0
#define SSW_DEC 0x0008
#define SSW_OFF 0x0008

#define SSW_SHIFT_MASK	0x0007
#define SSW_SHIFT_SHIFT 0
#define SSW_SHIFT(n) (n << SSW_SHIFT_SHIFT)

#define SSW_TIME_MASK 0x0070
#define SSW_TIME_SHIFT 4
#define SSW_TIME(n) (n << SSW_TIME_SHIFT)

#define SSW_BUILD(shift, dir, time) \
    (((time & 7) << 4) | (dir << 3) | (shift & 7))

// Sound square wave generator flags
#define SSQR_DUTY1_8 0
#define SSQR_DUTY1_4 0x0040
#define SSQR_DUTY1_2 0x0080
#define SSQR_DUTY3_4 0x00C0
#define SSQR_INC 0
#define SSQR_DEC 0x0800

#define SSQR_LEN_MASK 0x003F
#define SSQR_LEN_SHIFT 0
#define SSQR_LEN(n) (n << SSQR_LEN_SHIFT)

#define SSQR_DUTY_MASK 0x00C0
#define SSQR_DUTY_SHIFT 6
#define SSQR_DUTY(n) (n << SSQR_DUTY_SHIFT)

#define SSQR_TIME_MASK 0x0700
#define SSQR_TIME_SHIFT 8
#define SSQR_TIME(n) (n << SSQR_TIME_SHIFT)

#define SSQR_IVOL_MASK 0xF000
#define SSQR_IVOL_SHIFT 12
#define SSQR_IVOL(n) (n << SSQR_IVOL_SHIFT)

#define SSQR_ENV_BUILD(ivol, dir, time) \
    ((ivol << 12) | (dir << 11) | ((time & 7) << 8))

#define SSQR_BUILD(_ivol, dir, step, duty, len) \
    (SSQR_ENV_BUILD(ivol, dir, step) | ((duty & 3) << 6) | (len & 63))

// Sound square wave channel 2
#define REG_SND2CNT *(vu16*) (REG_BASE+0x0068)
#define REG_SND2FREQ *(vu16*) (REG_BASE+0x006C)

// Sound wave sound channel
#define REG_SND3SEL *(vu16*)(REG_BASE+0x0070)
#define REG_SND3CNT *(vu16*)(REG_BASE+0x0072)
#define REG_SND3FREQ *(vu16*)(REG_BASE+0x0074)

// Sound noise sound channel
#define REG_SND4CNT *(vu16*)(REG_BASE+0x0078)
#define REG_SND4FREQ *(vu16*)(REG_BASE+0x007C)

// Sound rates
typedef enum
{
    NOTE_C=0, NOTE_CIS, NOTE_D,   NOTE_DIS,
    NOTE_E,   NOTE_F,   NOTE_FIS, NOTE_G,
    NOTE_GIS, NOTE_A,   NOTE_BES, NOTE_B
} eSndNoteId;

const u32 __snd_rates[12]=
{
    8013, 7566, 7144, 6742, // C , C#, D , D#
    6362, 6005, 5666, 5346, // E , F , F#, G
    5048, 4766, 4499, 4246  // G#, A , A#, B
};

#define SND_RATE(note, oct) ( 2048-(__snd_rates[note]>>(4+(oct))) )

// Timing
#define FPS 60

// Player position
#define PLAYER_VIEW_OFFSET_MIN 8
#define PLAYER_VIEW_OFFSET_MAX 52
#define PLAYER_START 2
#define PLAYER_INTRO_LENGTH 70

// HUD
#define MAP_TOP 100
#define CURSOR_TOP 103
#define LEVEL_TOP 153
#define RECORDS_TOP 153
#define LEVEL_LEFT 110
#define MAP_LENGTH 240
#define TIME_DISPLAY_INDENT 2
#define HUD_SPACE 3
#define HUD_PADDING 3
#define CURRENT_TIME_LEFT (SCREEN_WIDTH - (GLYPH0_WIDTH * 5) - GLYPHCOLON_WIDTH - GLYPHPERIOD_WIDTH - \
                           HUD_PADDING - TIME_DISPLAY_INDENT - 7)
#define RECORDS_BORDER_TOP (RECORDS_TOP - HUD_PADDING)
#define LEVEL_BORDER_LEFT (LEVEL_LEFT - HUD_PADDING - 1)
#define LEVEL_BORDER_RIGHT (LEVEL_LEFT + LEVEL_WIDTH + HUD_SPACE + GLYPH0_WIDTH + HUD_PADDING)

// Explosion
#define EXPLOSION_LENGTH (FPS * 3)

// Cells
#define CELL_COUNT 720
#define CELL_SIZE 2
#define CELL_STEP (CELL_SIZE * 2)
#define LINE_TOP 76 // (SCREEN_HEIGHT / 2 - (CELL_SIZE * 4) / 2)
#define LINE_BOTTOM 84 // +8
#define CELLS_PER_SCREEN (SCREEN_WIDTH / (CELL_SIZE * 2))
#define STATE_EMPTY 0x0
#define STATE_PLAYER 0x1
#define STATE_ENEMY 0x2
#define STATE_GOAL 0x3
#define STATE_START 0x4
#define STATE_STREAM 0x5
#define STATE_LAVA 0x6
#define STATE_BOSS 0x7
#define STATE_ATTACK 0x8
#define STATE_CELEBRATE 0x9
#define STATE_DIE 0xA

// Goal
#define GOAL_LENGTH 4
#define GOAL_PALETTE_STEP 1
#define GOAL_CELEBRATION_MIN 4
#define GOAL_CELEBRATION_MAX 719
#define GOAL_CELEBRATION_STEP 3
#define GOAL_START (CELL_COUNT - GOAL_LENGTH)

// Screen
#define SCREEN_WIDTH 240
#define SCREEN_HEIGHT 160

// Colors
#define PALETTE_LENGTH 12
#define COLOR_BLACK RGB15(0, 0, 0)
#define COLOR_RED RGB15(31, 0, 0)
#define COLOR_ORANGE RGB15(31, 15, 0)
#define COLOR_YELLOW RGB15(31, 31, 0)
#define COLOR_GREEN RGB15(0, 31, 0)
#define COLOR_WHITE RGB15(31, 31, 31)

// Lava
#define LAVA_COLOR_SUNK COLOR_WHITE
#define LAVA_COLOR_RISING RGB15(31, 25, 15)
#define LAVA_COLOR_RISEN RGB15(31, 18, 0)

// Enemy
#define SPAWN_INTERVAL_LVL6 220
#define ENEMY_MAX_COUNT 32
#define BOSS_COLOR RGB15(31, 0, 25)
#define BOSS_LENGTH 5
#define BOSS_STEP 3

// Pause
#define PAUSE_MENU_TOP 121
#define PAUSE_RESUME_LEFT 55
#define PAUSE_RESET_LEFT 151
#define PAUSE_CURSOR_TOP 133
#define PAUSE_CURSOR_RESUME_LEFT 70
#define PAUSE_CURSOR_RESET_LEFT 163

// Attack
#define ATTACK_REACH 16
#define MAP_ATTACK_REACH 5

// Save
#define SAVE_OFFSET 16

// Title Menu
#define MODE_ORIGINAL 0
#define MODE_ADVANCE 1
#define TITLE_MENU_TOP 90
#define TITLE_MENU_ORIGINAL_LEFT 55
#define TITLE_MENU_ADVANCE_LEFT 136
#define TITLE_MENU_CURSOR_TOP 112
#define TITLE_MENU_CURSOR_ORIGINAL_LEFT 76
#define TITLE_MENU_CURSOR_ADVANCE_LEFT 154

/***
  
 start       if loop is set, this is the start boundary
 end         if loop is set, this is a boundary
 head        cell index of the enemy front*
 tail        cell index of the enemy back*
 frequency   position updates every Nth frame
 step        enemy head and tail move N cells every frame
 loop        1 = goes back and forth, 0 = goes straight forever
 alive       probably should just be set to 1
 direction   -1 = moves to the left, 1 = moves to the right

 * head and tail are the same value if the enemy is 1 cell long
  
 ***/
struct Enemy
{
    u16 start; 
    u16 end; 
    int head;
    int tail;
    u8 frequency;
    u8 step;
    u8 loop;
    u8 alive;
    int direction;
};

/***

 start        left edge cell index
 end          right edge cell index
 update_mod   see combinations below*
 step         see combinations below*
 offset       probably should be set to zero
  
 * fast = update_mod: 3, step: +/- 3
   medium = update_mod: 1, step: +/- 1
   slow = update_mod: 3, step: +/- 1
    
 ***/
struct Stream
{
    u16 start;
    u16 end;
    u8 update_mod;
    int step;
    u8 offset;
};

/***

 start     left edge cell index
 end       right edge cell index
 cycle     time in frame count for full cycle of lava*
 time      frame index in cycle where lava is currently
 split     should be cycle / 3
 color_ii  color index of lava**

 * gba runs at ~60fps, so 3s = 180, or 3 * FPS
 ** 0 (sunken), 1 (rising) or 2 (risen), should be based on time value

 ***/
struct Lava
{
    u16 start;
    u16 end;
    u16 cycle;
    u16 time;
    u16 split;
    u8 color_ii;
};

struct Boss
{
    u16 head;
    u16 tail;
    u8 phase;
    int moving;
};

struct Save
{
    int best_time_original;
    int best_time_advance;
    u8 advance_unlocked;
};

struct Time
{
    u8 min;
    u8 s;
    u8 ms;
};

static inline COLOR RGB15(u32 red, u32 green, u32 blue)
{
    return red | (green << 5) | (blue << 10);
}

static inline void vid_vsync(void)
{
    while (REG_VCOUNT >= 160);
    while (REG_VCOUNT < 160);
}

void blit(const u16*, u16, u16, u16, u16, u16);
void erase(u16, u16, u16, u16);
void explode(u8*, int*, int);
void reset_level(int*, struct Enemy (*)[], int*, struct Stream (*)[], u8*,
                 struct Lava (*)[], u8*, u8*, u8*, u8*, u8, u16*, int*, int*,
                 struct Boss*, struct Save*, int, u8);
void draw_cell(int, COLOR);
u16 rotl16(u16, u8);
u32 rotl32(u32, u8);
void bytecpy(void*, const void*, u32);
void horizontal_line(u16, u16, u16, COLOR);
void vertical_line(u16, u16, u16, COLOR);
void draw_best_time(int);
void draw_current_time_border();
void draw_current_time(int);
void erase_current_time();
void draw_level_border();
void erase_level();
void erase_line();
void draw_title_menu_options();
void draw_title_menu_cursor(u8);
void parse_time(int, struct Time*);
void draw_time(struct Time*, int, int);
void draw_digits(int, int, int, int);
u8 count_digits(int);
const u16* get_glyph(u8);
